@@ -161,7 +161,7 @@ static MagickBool IsHEIF(const unsigned char *magick,const size_t length)
   /*
     const char* heif_get_file_mime_type(const uint8_t* data, int len)
     heif_filetype_result heif_check_filetype(const uint8_t* data, int len)
-   */
+  */
 
   if (length < 12)
     return(MagickFalse);
@@ -212,6 +212,8 @@ static MagickBool IsHEIF(const unsigned char *magick,const size_t length)
         heif_image_handle_release(heif_image_handle);                  \
       if (heif)                                                        \
         heif_context_free(heif);                                       \
+      if (decode_options)                                              \
+        heif_decoding_options_free(decode_options);                    \
       MagickFreeResourceLimitedMemory(in_buf);                         \
     } while (0);
 
@@ -473,8 +475,8 @@ static MagickPassFail ReadColorProfile(struct heif_image_handle *heif_image_hand
           SetImageProfile(image,"ICM",profile,profile_size);
           MagickFreeResourceLimitedMemory(profile);
           if (image->logging)
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                              "  done reading ICC profile");
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "  done reading ICC profile");
         }
     }
   return MagickPass;
@@ -569,15 +571,6 @@ static MagickPassFail apply_security_limits(const ImageInfo *image_info,
   memory_limit = (magick_uintmax_t) GetMagickResourceLimit(MemoryResource);
   security_limits->max_memory_block_size = (uint64_t) memory_limit;
   security_limits->max_total_memory = (uint64_t) memory_limit;
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_memory_block_size = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_memory_block_size);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_total_memory = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_total_memory);
-
   width_limit = (magick_uintmax_t) GetMagickResourceLimit(WidthResource);
   if (width_limit > INT_MAX)
     width_limit = INT_MAX;
@@ -590,10 +583,6 @@ static MagickPassFail apply_security_limits(const ImageInfo *image_info,
     pixels_limit = width_limit*height_limit;
 
   security_limits->max_image_size_pixels = (uint64_t) pixels_limit;
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_image_size_pixels = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_image_size_pixels);
 
   /*
     Version 1 interface has:
@@ -623,99 +612,81 @@ static MagickPassFail apply_security_limits(const ImageInfo *image_info,
   /* version 1 interface */
   MagickAttributeToU64(image_info,"heif","max-image-size-pixels",
                        security_limits->max_image_size_pixels);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_image_size_pixels = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_image_size_pixels);
 
   MagickAttributeToU64(image_info,"heif","max-number-of-tiles",
                        security_limits->max_number_of_tiles);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_number_of_tiles = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_number_of_tiles);
 
   MagickAttributeToU32(image_info,"heif","max-bayer-pattern-pixels",
                        security_limits->max_bayer_pattern_pixels);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_bayer_pattern_pixels = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_bayer_pattern_pixels);
 
   MagickAttributeToU32(image_info,"heif","max-items",
                        security_limits->max_items);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_items = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_items);
 
   MagickAttributeToU32(image_info,"heif","max-color-profile-size",
                        security_limits->max_color_profile_size);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_color_profile_size = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_color_profile_size);
 
   MagickAttributeToU64(image_info,"heif","max-memory-block-size",
                        security_limits->max_memory_block_size);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_memory_block_size = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_memory_block_size);
 
   MagickAttributeToU32(image_info,"heif","max-components",
                        security_limits->max_components);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_components = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_components);
 
   MagickAttributeToU32(image_info,"heif","max-iloc-extents-per-item",
                        security_limits->max_iloc_extents_per_item);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_iloc_extents_per_item = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_iloc_extents_per_item);
 
   MagickAttributeToU32(image_info,"heif","max-size-entity-group",
                        security_limits->max_size_entity_group);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_size_entity_group = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_size_entity_group);
 
   MagickAttributeToU32(image_info,"heif","max-children-per-box",
                        security_limits->max_children_per_box);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_children_per_box = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_children_per_box);
 
   /* version 2 interface */
 
   /* Added in 1.13.0, September 2, 2022 */
   MagickAttributeToU64(image_info,"heif","max-total-memory",
                        security_limits->max_total_memory);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_total_memory = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_total_memory);
 
   /* Added in 1.13.0, September 2, 2022 */
   MagickAttributeToU32(image_info,"heif","max-sample-description-box-entries",
                        security_limits->max_sample_description_box_entries);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "security limits:"
-                        "  max_sample_description_box_entries = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_sample_description_box_entries);
 
   /* Added in 1.17.6? ?*/
   MagickAttributeToU32(image_info,"heif","max-sample-group-description-box-entries",
                        security_limits->max_sample_group_description_box_entries);
+
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                         "security limits:"
+                        /* version 1 interface */
+                        "  max_image_size_pixels = %" MAGICK_UINTMAX_F "u"","
+                        "  max_number_of_tiles = %" MAGICK_UINTMAX_F "u"","
+                        "  max_bayer_pattern_pixels = %" MAGICK_UINTMAX_F "u"","
+                        "  max_items = %" MAGICK_UINTMAX_F "u"","
+                        "  max_color_profile_size = %" MAGICK_UINTMAX_F "u"","
+                        "  max_memory_block_size = %" MAGICK_UINTMAX_F "u"","
+                        "  max_components = %" MAGICK_UINTMAX_F "u"","
+                        "  max_iloc_extents_per_item = %" MAGICK_UINTMAX_F "u"","
+                        "  max_size_entity_group = %" MAGICK_UINTMAX_F "u"","
+                        "  max_children_per_box = %" MAGICK_UINTMAX_F "u"","
+                        /* version 2 interface */
+                        "  max_total_memory = %" MAGICK_UINTMAX_F "u"","
+                        "  max_sample_description_box_entries = %" MAGICK_UINTMAX_F "u"","
                         "  max_sample_group_description_box_entries = %" MAGICK_UINTMAX_F "u",
-                        (magick_uintmax_t) security_limits->max_sample_group_description_box_entries);
+                        /* version 1 interface */
+                        (magick_uintmax_t) security_limits->max_image_size_pixels,
+                        (magick_uintmax_t) security_limits->max_number_of_tiles,
+                        (magick_uintmax_t) security_limits->max_bayer_pattern_pixels,
+                        (magick_uintmax_t) security_limits->max_items,
+                        (magick_uintmax_t) security_limits->max_color_profile_size,
+                        (magick_uintmax_t) security_limits->max_memory_block_size,
+                        (magick_uintmax_t) security_limits->max_components,
+                        (magick_uintmax_t) security_limits->max_iloc_extents_per_item,
+                        (magick_uintmax_t) security_limits->max_size_entity_group,
+                        (magick_uintmax_t) security_limits->max_children_per_box,
+                        /* version 2 interface */
+                        (magick_uintmax_t) security_limits->max_total_memory,
+                        (magick_uintmax_t) security_limits->max_sample_description_box_entries,
+                        (magick_uintmax_t) security_limits->max_sample_group_description_box_entries
+                        );
 #else
   {
     /* DEPRECATED: Add an image size limit based on maximum_width ^ 2 */
@@ -875,16 +846,16 @@ static const char *HEIF_channel_to_string(const enum heif_channel c)
 }
 
 static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
-                                         MagickBool ignore_transformations,
+                                         struct heif_decoding_options *decode_options,
                                          Image *image,
                                          const ImageInfo *image_info,
                                          ExceptionInfo *exception)
 {
-#define HEIFReadImageFrameCleanup()                                    \
-  do                                                                   \
-    {                                                                  \
-      if (heif_image)                                                  \
-        heif_image_release(heif_image);                                \
+#define HEIFReadImageFrameCleanup()             \
+  do                                            \
+    {                                           \
+      if (heif_image)                           \
+        heif_image_release(heif_image);         \
     } while (0);
 
 #define ThrowHEIFThrowReadImageFrameException(code_,reason_,image_)     \
@@ -894,7 +865,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
       if (code_ > exception->severity)                                  \
         {                                                               \
           ThrowException(&(image_)->exception,code_,reason_,(image_)->filename); \
-          CopyException(exception,&(image_)->exception);               \
+          CopyException(exception,&(image_)->exception);                \
         }                                                               \
       return(MagickFail);                                               \
     } while (0);
@@ -902,9 +873,6 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
   struct heif_image
     *heif_image = NULL;
 
-  struct heif_decoding_options
-    *decode_options;
-
   PixelPacket
     *q;
 
@@ -946,7 +914,11 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
   if (image_info->ping)
     {
       bits_per_pixel=heif_image_handle_get_luma_bits_per_pixel(heif_image_handle);
-      if (bits_per_pixel != -1)
+      if (image->logging)
+        if (bits_per_pixel < 0)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                "Image has undefined bit-depth!");
+      if (bits_per_pixel > 0)
         image->depth=(unsigned int) bits_per_pixel;
       if (image->logging)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -983,7 +955,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
                           HEIF_chroma_to_string(preferred_chroma));
 
   /* Read EXIF and XMP profile */
-  if (!ReadMetadata(heif_image_handle, image, ignore_transformations))
+  if (!ReadMetadata(heif_image_handle, image, decode_options->ignore_transformations))
     {
       CopyException(exception,&image->exception);
       HEIFReadImageFrameCleanup();
@@ -999,9 +971,9 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
     }
 
   /*
-    When apply transformations the whole image has to be read to get
-    the real dimensions. We no longer read the whole image because it
-    makes 'identify' extremely slow for large files.
+    When applying transformations the whole image has to be read to
+    get the real dimensions. We no longer read the whole image because
+    it makes 'identify' extremely slow for large files.
 
     The user should specify:
 
@@ -1010,7 +982,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
     in order to force reading much of the image to get fully accurate
     information.
   */
-  if (image_info->ping && !ignore_transformations)
+  if (image_info->ping && decode_options->ignore_transformations)
     {
       HEIFReadImageFrameCleanup();
       if (image->logging)
@@ -1019,40 +991,6 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
       return MagickPass;
     }
 
-  /* Add decoding options support */
-  decode_options = heif_decoding_options_alloc();
-  if (decode_options == (struct heif_decoding_options*) NULL)
-    ThrowHEIFThrowReadImageFrameException(ResourceLimitError,MemoryAllocationFailed,image);
-
-#if HEIF_ENABLE_PROGRESS_MONITOR
-  progress_user_data.exception = exception;
-  progress_user_data.image = image;
-  progress_user_data.max_progress = 0;
-  progress_user_data.progress = 0;
-#endif
-
-  decode_options->ignore_transformations = ignore_transformations;
-
-#if HEIF_ENABLE_PROGRESS_MONITOR
-  decode_options->start_progress = start_progress;
-  decode_options->on_progress = on_progress;
-  decode_options->end_progress = end_progress;
-  decode_options->progress_user_data = &progress_user_data;
-#endif /* if HEIF_ENABLE_PROGRESS_MONITOR */
-
-  /* version 2 decode options */
-
-  /* Let libheif do the scaling from deep images if QuantumDepth == 8 */
-#if QuantumDepth == 8
-  decode_options->convert_hdr_to_8bit = 1;
-#endif /* if QuantumDepth == 8 */
-
-  /* version 3 decode options */
-
-  /* When enabled, an error is returned for invalid input. Otherwise, it will try its best and
-     add decoding warnings to the decoded heif_image. Default is non-strict. */
-  /* uint8_t strict_decoding; */
-
 
   /*
     Be prepared to handle images on per-plane basis. Upscale
@@ -1098,13 +1036,65 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
       }
 #endif
 
+    if (image->logging && IsEventLogged(CoderEvent))
+      {
+#if 1
+        /* See libheif/api/libheif/heif_tiling.h */
+        heif_image_tiling tiling;
+        (void) memset(&tiling,0,sizeof(tiling));
+        heif_status=heif_image_handle_get_image_tiling(heif_image_handle, decode_options->ignore_transformations, &tiling);
+        if (heif_status.code != heif_error_Ok)
+          {
+            if (image->logging)
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                    "heif_image_handle_get_image_tiling() reports error \"%s\"",
+                                    heif_status.message);
+            ThrowHEIFThrowReadImageFrameException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
+          }
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Tiling: version %d, num_columns=%u, num_rows=%u,"
+                              " tile_width=%u, tile_height=%u, image_width=%u, image_height=%u,"
+                              " top_offset=%u, left_offset=%u, number_of_extra_dimensions=%u",
+                              tiling.version, tiling.num_columns, tiling.num_rows,
+                              tiling.tile_width, tiling.tile_height, tiling.image_width, tiling.image_height,
+                              tiling.top_offset, tiling.left_offset,
+                              tiling.number_of_extra_dimensions);
+#endif
+#if 0
+        /* From heif_dec.cc decode_image_tiles() */
+        for (uint32_t ty = 0; ty < tiling.num_rows; ty++)
+          for (uint32_t tx = 0; tx < tiling.num_columns; tx++)
+            {
+              /*
+                heif_image* image;
+                heif_error err;
+                err = heif_image_handle_decode_image_tile(handle,
+                &image,
+                encoder->colorspace(has_alpha),
+                encoder->chroma(has_alpha, bit_depth),
+                decode_options, tx, ty);
+                heif_error heif_image_handle_decode_image_tile(const heif_image_handle* in_handle,
+                heif_image** out_img,
+                enum heif_colorspace colorspace,
+                enum heif_chroma chroma,
+                const heif_decoding_options* options,
+                uint32_t tile_x, uint32_t tile_y);
+                heif_image_release(image);
+              */
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                    "Tile %u,%u: (x1=%u y1=%u x2=%u y2=%u)",
+                                    tx, ty, tx*tiling.tile_width, ty*tiling.tile_height,
+                                    ((tx+1)*tiling.tile_width)-1, ((ty+1)*tiling.tile_height)-1);
+            }
+#endif
+      }
+
     heif_status=heif_decode_image(heif_image_handle,
                                   &heif_image,
                                   heif_colorspace_decode, /* Chose preferred colorspace */
                                   heif_chroma_decode,
                                   decode_options);
   }
-  heif_decoding_options_free(decode_options);
   if (heif_status.code == heif_error_Memory_allocation_error)
     ThrowHEIFThrowReadImageFrameException(ResourceLimitError,MemoryAllocationFailed,image);
   if (heif_status.code != heif_error_Ok)
@@ -1120,13 +1110,13 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
       if (image->logging)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                               "heif_decode_image() reports heif_error_Ok but failed to return an image!");
-          ThrowHEIFThrowReadImageFrameException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
+      ThrowHEIFThrowReadImageFrameException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
     }
 
   /*
     Trace decoding warnings
   */
-  if (image->logging)
+  if (image->logging && IsEventLogged(CoderEvent))
     {
       int i;
 
@@ -1166,7 +1156,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
       return MagickPass;
     }
 
-    if (CheckImagePixelLimits(image, exception) != MagickPass)
+  if (CheckImagePixelLimits(image, exception) != MagickPass)
     {
       HEIFReadImageFrameCleanup();
       return MagickFail;
@@ -1203,7 +1193,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
             line[4] = { 0, 0, 0, 0 };
 
           size_t
-            row_stride;
+            row_stride[4];
 
           unsigned int
             channel_index,
@@ -1224,7 +1214,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
               heif_channel = channels_YCbCrA[channel_index];
               planes[channel_index] = heif_image_get_plane_readonly2(heif_image,
                                                                      heif_channel,
-                                                                     &row_stride);
+                                                                     &row_stride[channel_index]);
               heif_channel_str = HEIF_channel_to_string(heif_channel);
               bits_per_pixel = heif_image_get_bits_per_pixel(heif_image,
                                                              heif_channel);
@@ -1236,7 +1226,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
               if (image->logging)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                       "  channel %s row stride %zu (%d bits in %d bit quantum)",
-                                      heif_channel_str, row_stride, bits_per_pixel_range, bits_per_pixel);
+                                      heif_channel_str, row_stride[channel_index], bits_per_pixel_range, bits_per_pixel);
             }
           max_value_given_bits=MaxValueGivenBits(image->depth);
 
@@ -1253,7 +1243,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
                   return MagickFail;
                 }
               for (channel_index = 0; channel_index < num_channels; channel_index++)
-                line[channel_index]=planes[channel_index]+y*row_stride;
+                line[channel_index]=planes[channel_index]+y*row_stride[channel_index];
               for (x=0; x < (long)image->columns; x++)
                 {
                   for (channel_index = 0; channel_index < num_channels; channel_index++)
@@ -1322,7 +1312,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
             line[4] = { 0, 0, 0 , 0 };
 
           size_t
-            row_stride;
+            row_stride[4];
 
           unsigned int
             channel_index,
@@ -1344,7 +1334,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
               heif_channel = channels_RGBA[channel_index];
               planes[channel_index] = heif_image_get_plane_readonly2(heif_image,
                                                                      heif_channel,
-                                                                     &row_stride);
+                                                                     &row_stride[channel_index]);
               heif_channel_str = HEIF_channel_to_string(heif_channel);
               bits_per_pixel = heif_image_get_bits_per_pixel(heif_image,
                                                              heif_channel);
@@ -1356,7 +1346,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
               if (image->logging)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                       "  channel %s row stride %zu (%d bits in %d bit quantum)",
-                                      heif_channel_str, row_stride, bits_per_pixel_range, bits_per_pixel);
+                                      heif_channel_str, row_stride[channel_index], bits_per_pixel_range, bits_per_pixel);
             }
           max_value_given_bits=MaxValueGivenBits(image->depth);
 
@@ -1373,7 +1363,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
                   return MagickFail;
                 }
               for (channel_index = 0; channel_index < num_channels; channel_index++)
-                line[channel_index]=planes[channel_index]+y*row_stride;
+                line[channel_index]=planes[channel_index]+y*row_stride[channel_index];
               for (x=0; x < (long)image->columns; x++)
                 {
                   for (channel_index = 0; channel_index < num_channels; channel_index++)
@@ -1442,7 +1432,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
             line[2] = { 0, 0 };
 
           size_t
-            row_stride;
+            row_stride[2];
 
           unsigned int
             channel_index,
@@ -1465,7 +1455,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
               heif_channel = channels_YA[channel_index];
               planes[channel_index] = heif_image_get_plane_readonly2(heif_image,
                                                                      heif_channel,
-                                                                     &row_stride);
+                                                                     &row_stride[channel_index]);
               bits_per_pixel = heif_image_get_bits_per_pixel(heif_image,
                                                              heif_channel);
               bits_per_pixel_range = heif_image_get_bits_per_pixel_range(heif_image,
@@ -1477,7 +1467,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
               if (image->logging)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                       "  channel %s row stride %zu (%d bits in %d bit quantum)",
-                                      heif_channel_str, row_stride, bits_per_pixel_range, bits_per_pixel);
+                                      heif_channel_str, row_stride[channel_index], bits_per_pixel_range, bits_per_pixel);
             }
           max_value_given_bits=MaxValueGivenBits(image->depth);
 
@@ -1494,7 +1484,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
                   return MagickFail;
                 }
               for (channel_index = 0; channel_index < num_channels; channel_index++)
-                line[channel_index]=planes[channel_index]+y*row_stride;
+                line[channel_index]=planes[channel_index]+y*row_stride[channel_index];
               for (x=0; x < (long)image->columns; x++)
                 {
                   for (channel_index = 0; channel_index < num_channels; channel_index++)
@@ -1574,6 +1564,9 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
   struct heif_image
     *heif_image = NULL;
 
+  struct heif_decoding_options
+    *decode_options = NULL;
+
   const char
     *value;
 
@@ -1591,9 +1584,6 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
     progress_user_data;
 #endif
 
-  MagickBool
-    ignore_transformations = image_info->ping ? MagickTrue : MagickFalse;
-
   MagickPassFail
     status;
 
@@ -1619,6 +1609,37 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
       heif_initialized = MagickTrue;
     }
 
+
+  /* Add decoding options support */
+  decode_options = heif_decoding_options_alloc();
+  if (decode_options == (struct heif_decoding_options*) NULL)
+    ThrowHEIFReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+
+  decode_options->ignore_transformations = image_info->ping ? MagickTrue : MagickFalse;
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "Initial ignore_transformations = %s", decode_options->ignore_transformations ? "True" : "False");
+#if HEIF_ENABLE_PROGRESS_MONITOR
+  progress_user_data.exception = exception;
+  progress_user_data.image = image;
+  progress_user_data.max_progress = 0;
+  progress_user_data.progress = 0;
+#endif
+
+#if HEIF_ENABLE_PROGRESS_MONITOR
+  decode_options->start_progress = start_progress;
+  decode_options->on_progress = on_progress;
+  decode_options->end_progress = end_progress;
+  decode_options->progress_user_data = &progress_user_data;
+#endif /* if HEIF_ENABLE_PROGRESS_MONITOR */
+
+  /* version 2 decode options */
+
+  /* Let libheif do the scaling from deep images if QuantumDepth == 8 */
+#if QuantumDepth == 8
+  decode_options->convert_hdr_to_8bit = 1;
+#endif /* if QuantumDepth == 8 */
+
+
   /*
     Read the whole input file into a memory blob
   */
@@ -1645,7 +1666,7 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
   /*
     Trace some information about the file
   */
-  if (image->logging && IsEventLogged(CoderEvent))
+  if (image->logging)
     {
       char
         fourcc[5];
@@ -1719,14 +1740,15 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
   if ((value=AccessDefinition(image_info,"heif","ignore-transformations")))
     {
       if (LocaleCompare(value,"FALSE") == 0)
-        ignore_transformations = MagickFalse;
+        decode_options->ignore_transformations = MagickFalse;
       else if (LocaleCompare(value,"TRUE") == 0)
-        ignore_transformations = MagickTrue;
+        decode_options->ignore_transformations = MagickTrue;
+      if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Updated ignore-transformations: %s",
+                              decode_options->ignore_transformations ? "True" : "False");
     }
 
-  if (image->logging)
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "ignore-transformations: %s", ignore_transformations ? "True" : "False");
 
   heif_status=heif_context_read_from_memory_without_copy(heif, in_buf, in_len, NULL);
   if (heif_status.code == heif_error_Unsupported_filetype
@@ -1766,7 +1788,7 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
       }
   }
 
-  status=ReadHEIFImageFrame(heif_image_handle,ignore_transformations,image,image_info,exception);
+  status=ReadHEIFImageFrame(heif_image_handle,decode_options,image,image_info,exception);
 
   HEIFReadCleanup();
   CloseBlob(image);
