@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2008-2022 GraphicsMagick Group
+% Copyright (C) 2008-2025 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -114,6 +114,8 @@ MagickExport ChannelType StringToChannelType(const char *option)
     channel=YellowChannel;
   else if (LocaleCompare("Opacity",option) == 0)
     channel=OpacityChannel;
+  else if (LocaleCompare("Alpha",option) == 0) /* ImageMagick compatibility */
+    channel=OpacityChannel;
   else if (LocaleCompare("Black",option) == 0)
     channel=BlackChannel;
   else if (LocaleCompare("Matte",option) == 0)
@@ -460,6 +462,7 @@ MagickExport CompositeOperator StringToCompositeOperator(const char *option)
         { CopyGreenCompositeOp, "CopyGreen" },
         { CopyBlueCompositeOp, "CopyBlue" },
         { CopyOpacityCompositeOp, "CopyOpacity" },
+        { CopyOpacityCompositeOp, "CopyAlpha" }, /* ImageMagick compatibility */
         { ClearCompositeOp, "Clear" },
         { DissolveCompositeOp, "Dissolve" },
         { DisplaceCompositeOp, "Displace" },
@@ -628,7 +631,9 @@ MagickExport CompressionType StringToCompressionType(const char *option)
     compression_type=Group4Compression;
   else if (LocaleCompare("JPEG",option) == 0)
     compression_type=JPEGCompression;
-  else if (LocaleCompare("Lossless",option) == 0)
+  else if (LocaleCompare("LosslessJPEG",option) == 0)
+    compression_type=LosslessJPEGCompression;
+  else if (LocaleCompare("Lossless",option) == 0) /* Bad choice! */
     compression_type=LosslessJPEGCompression;
   else if (LocaleCompare("LZW",option) == 0)
     compression_type=LZWCompression;
@@ -1662,6 +1667,8 @@ MagickExport ResourceType StringToResourceType(const char *option)
     resource_type=HeightResource;
   else if (LocaleCompare("Read",option) == 0)
     resource_type=ReadResource;
+  else if (LocaleCompare("Write",option) == 0)
+    resource_type=WriteResource;
   return resource_type;
 }
 
